@@ -391,7 +391,7 @@ para = do
 --
 
 bibliographic :: GenParser Char ParserState Block
-bibliographic = choice [ maketitle, title, authors, date ]
+bibliographic = choice [ maketitle, title, subtitle, authors, date ]
 
 maketitle :: GenParser Char st Block
 maketitle = try (string "\\maketitle") >> spaces >> return Null
@@ -404,11 +404,22 @@ title = try $ do
   updateState (\state -> state { stateTitle = tit })
   return Null
 
+subtitle :: GenParser Char ParserState Block
+subtitle = try $ do
+  string "\\subtitle{"
+  tit <- manyTill inline (char '}')
+  spaces
+  updateState (\state -> state { stateTitle = stateTitle state ++
+                                   Str ":" : LineBreak : tit })
+  return Null
+
 authors :: GenParser Char ParserState Block
 authors = try $ do
   string "\\author{"
-  raw <- many1 (notFollowedBy (char '}') >> inline)
-  let authors' = map normalizeSpaces $ splitBy (== LineBreak) raw
+  let andsep = try $ string "\\and" >> notFollowedBy letter >>
+                     spaces >> return '&'
+  raw <- sepBy (many $ notFollowedBy (char '}' <|> andsep) >> inline) andsep
+  let authors' = map normalizeSpaces raw
   char '}'
   spaces
   updateState (\s -> s { stateAuthors = authors' })
@@ -777,8 +788,12 @@ doubleQuoteEnd :: CharParser st String
 doubleQuoteEnd = try $ string "''"
 
 ellipses :: GenParser Char st Inline
-ellipses = try $ string "\\ldots" >> optional (try $ string "{}") >>
-                 return Ellipses
+ellipses = try $ do
+  char '\\'
+  optional $ char 'l'
+  string "dots"
+  optional $ try $ string "{}"
+  return Ellipses
 
 enDash :: GenParser Char st Inline
 enDash = try (string "--") >> return EnDash
@@ -801,7 +816,11 @@ nonbreakingSpace = char '~' >> return (Str "\160")
 
 -- hard line break
 linebreak :: GenParser Char st Inline
-linebreak = try (string "\\\\") >> return LineBreak
+linebreak = try $ do
+  string "\\\\"
+  optional $ bracketedText '[' ']'  -- e.g. \\[10pt]
+  spaces
+  return LineBreak
 
 str :: GenParser Char st Inline
 str = many1 (noneOf specialChars) >>= return . Str
