@@ -67,7 +67,7 @@ module Text.Pandoc.Shared (
                      -- * File handling
                      inDirectory,
                      findDataFile,
-                     readDataFile
+                     readDataFile,
                     ) where
 
 import Text.Pandoc.Definition
@@ -493,6 +493,7 @@ data WriterOptions = WriterOptions
   , writerAscii            :: Bool       -- ^ Avoid non-ascii characters
   } deriving Show
 
+{-# DEPRECATED writerXeTeX "writerXeTeX no longer does anything" #-}
 -- | Default writer options.
 defaultWriterOptions :: WriterOptions
 defaultWriterOptions = 
